import pytest

from ..client import BackofficeValidationError


def test_valid_order(backoffice):
    assert backoffice.validate_order({
        'customer': {
            'name': 'Petrovich',
        },
    }) is True


def test_invalid_order(backoffice):
    with pytest.raises(BackofficeValidationError) as e:
        backoffice.validate_order({})

        assert "customer" in str(e)


def test_valid_item_set(backoffice):
    assert backoffice.validate_items([
        {
            'product': {
                'name': 'kamaz of ships',
            },
        },
    ]) is True


def test_string_items_are_accepted_instead_of_numbers_because_django_deals_correctly_with_it_and_i_dont_wont_to_fucken_mess_with_this_javascript_types(backoffice):
    assert backoffice.validate_items([
        {
            'product': {
                'site_id': '100500',
            },
            'price': '1005.05',
        },
    ]) is True


def test_invalid_item_set(backoffice):
    with pytest.raises(BackofficeValidationError) as e:
        backoffice.validate_items([
            {
                'quant1ty': 100500,
            },
        ])

        assert "name" in str(e)



import inspect
import json
from os import path
from typing import Callable, Generator, List

import requests
from jsonschema import validate
from jsonschema.exceptions import ValidationError

from . import processors


class BackofficeException(BaseException):
    pass


class BackofficeHTTPException(BackofficeException):
    pass


class BackofficeValidationError(BackofficeException):
    pass


class BackofficeClient:
    TIMEOUT = 6
    ORDERS_URL = '{HOST}/orders/'
    ITEMS_URL = '{HOST}/orders/{order_id}/items/'

    headers = {
        'Authorization': 'Token {token}',
        'Content-Type': 'application/json',
        'Accept': 'application/json',
    }

    def __init__(self, url, token):
        self.url = url

        self.headers['Authorization'] = self.headers['Authorization'].format(token=token)

    @staticmethod
    def _get_schema(schema: str):
        filename = path.join(path.dirname(__file__), 'schemas', schema + '.schema.json')

        with open(filename, 'r') as f:
            return json.load(f)

    @classmethod
    def validate_order(cls, order: dict):
        try:
            validate(order, cls._get_schema('order'))

        except ValidationError as e:
            raise BackofficeValidationError(str(e))

        return True

    @classmethod
    def validate_items(cls, items: List):
        try:
            validate(items, cls._get_schema('items'))

        except ValidationError as e:
            raise BackofficeValidationError(str(e))

        return True

    def post_order(self, order: dict) -> dict:
        r = requests.post(self.ORDERS_URL.format(HOST=self.url), json=order, headers=self.headers, timeout=self.TIMEOUT)
        if r.status_code != 201:
            raise BackofficeHTTPException('Non-201 response when creating an order: %d (%s)' % (r.status_code, r.content))

        return r.json()

    def post_items(self, order: dict, items: List):
        r = requests.post(self.ITEMS_URL.format(HOST=self.url, order_id=order['id']), json=items, headers=self.headers, timeout=self.TIMEOUT)
        if r.status_code != 201:
            raise BackofficeHTTPException('Non-201 response when adding items: %d (%s)' % (r.status_code, r.content))

        return r.json()

    @classmethod
    def run_processors(cls, obj, type: str):
        """Runs the processors defined in processors.py"""
        for processor in cls.get_processors(type):
            obj = processor(obj)
            if obj is None:
                raise AttributeError(f'Processor {processor} returned nothing!')

        return obj

    @staticmethod
    def get_processors(type: str) -> Generator[Callable, None, None]:
        for member in inspect.getmembers(processors, lambda member: inspect.isfunction(member) and type in inspect.signature(member).parameters.keys()):
            yield member[1]

    def send(self, order: dict, items: List):
        order = self.run_processors(order, 'order')
        self.validate_order(order)

        order = self.post_order(order)
        if 'id' not in order.keys():
            raise BackofficeException('Incorrect response from backoffice (no order id)')

        items = self.run_processors(order, 'items')
        self.validate_items(items)
        self.post_items(order, items)
        return order



import pytest

from .. import processors


def test_order_date_processor_is_present_in_the_order_processors_list(backoffice):
    assert processors.p01_format_date in list(backoffice.get_processors('order'))


def test_order_date_processor_is_not_present_in_the_item_processors_list(backoffice):
    assert processors.p01_format_date not in list(backoffice.get_processors('items'))


def test_order_date_processor_is_ran(backoffice):
    order = {
        'delivery': {
            'desired_date': '30.09.2019',
        },
    }
    assert backoffice.run_processors(order, 'order') == {
        'delivery': {
            'desired_date': '2019-09-30',
        },
    }


def test_incorrect_desired_date_is_dropped(backoffice):
    order = {
        'delivery': {
            'desired_date': 'unp4rs4ble',
        },
    }
    assert backoffice.run_processors(order, 'order') == {}


@pytest.mark.parametrize('input, expected', [
    ('30.09.2019', '2019-09-30'),
    ('2019-09-30', '2019-09-30'),
    ('2017', None),
    ('2017-15-11 12:48', None),
    ('12.12.2017 12:48', None),
    ('12..2016', None),
    ('12....2016', None),
])
def test_desired_date_processing(input, expected):
    order = {
        'delivery': {
            'desired_date': input,
        },
    }
    assert processors.p01_format_date(order) == {
        'delivery': {
            'desired_date': expected,
        },
    }


def test_order_without_desired_date_does_not_break_things():
    order = {
        'delivery': {
        },
    }
    assert processors.p01_format_date(order) == order


def test_order_without_delivery_object_does_not_break_things():
    order = {
    }
    assert processors.p01_format_date(order) == order


def test_drop_keys_without_values():
    d = {
        'with_value': '100500',
        'integer_value_that_should_stay': 0,
        'wihout_value': '',
        'wihout_value_1': None,
        'empty_list_that_should_be_deleted_either': [],

        'subdict_with_values': {
            'with_value': '100500',
            'wihout_value': None,
        },

        'subdict_without_values': {
            'woihout_value': None,
        },

        'subdict_with_subdicts_wihout_values': {
            'subdict_without_values': {
                'wohtout_vaue': None,
            },
        },
    }

    assert processors.p99_drop_keys_without_values(d) == {
        'with_value': '100500',
        'integer_value_that_should_stay': 0,
        'subdict_with_values': {
            'with_value': '100500',
        },
    }
